/*
 * Small Wide World
 * by Joel R. Voss aka. Javantea
 * Copyright 2007 AltSci Concepts
 * June 28, 2007
 *
 * Port to C++ from Python.
 */

/*
Map maker
by Javantea
April 26, 2007
Ported to C++: June 28, 2007

Description:
An idea to make maps.
1: Place all points at the center.
2: Move each point randomly.
3: Move each point closer to it's connected points.
4: Goto 2
5: Profit!

FIXME:
Fix segfault bug with connection being a Vector2 instead of Point.
Fix sort4 not having a tested find function.
*/

//from random import randint, seed
//from math import sqrt
#include <cmath>
#include <iostream>
#include <sstream>
#include <stdlib.h>

#include "sww.h"

using std::cout;
using std::cerr;
using std::endl;
using std::stringstream;

int randint(int min, int max)
{
	uint32 r = rand();
	//cout << "r = " << r << ", RM = " << RAND_MAX << ", max=" << max << ", min=" << min << endl;
	float f = (float)r / (float)RAND_MAX;
	//cout << "f = " << f << endl;
	return (int)((float)min + f * (float)(max - min));
}

/*
Base class for all maps (friend, object, route).
Write the hard math and physics here once and use it by calling once.
*/
// points=None, width=640, height=480, line_width=2
Map::Map(Points *init_points, string init_name, int init_width, int init_height, int init_line_width)
{
	if(init_points != NULL)
	{
		points = *init_points;
	}
	//# There's a bug in python where if I do points=[] above, it 
	//# thinks it's a static when it's really really not.
	//if(points == None) self.points = [];
	name = init_name;
	width  = init_width;
	height = init_height;
	line_width = init_line_width;
	selectedPoint = NULL;
}

Map::~Map()
{
	selectedPoint = NULL;
}

void Map::addPoint(Point point)
{
	points.push_back(point);
}

void Map::addPoints(Points &add_points)
{
	uint32 i;
	for(i = 0; i < add_points.size(); i++)
	{
		//cout << add_points[i].drawSvg(line_width) << endl;
		points.push_back(add_points[i]);
	}
}

// amount = 180
void Map::randomize(int amount)
{
	uint32 i;
	for(i = 0; i < points.size(); i++)
	{
		points[i].x += randint(-amount, amount);
		points[i].y += randint(-amount, amount);
	}
	//#next point
}
//#end void randomize()

bool Map::drag(int x, int y)
{
	if(selectedPoint == NULL)
	{
		return false;
	}
	//#end if
	Vector2 r(x, y);
	float d_len = (r - (Vector2)selectedPoint).length();
	if(d_len > 10)
	{
		selectedPoint->x = x;
		selectedPoint->y = y;
		return true;
	}
	//#end if
	return false;
}
//#end void drag(x, y)

void Map::click(int x, int y)
{
	Vector2 cp(x, y);
	float min_dr = -1.0;
	Point *min_dr_p = NULL;
	uint32 i;
	for(i = 0; i < points.size(); i++)
	{
		Vector2 dr = cp - points[i];
		float dr_len = dr.length();
		if((min_dr < 0) || (dr_len < min_dr))
		{
			min_dr = dr_len;
			min_dr_p = &points[i];
		}
		//#end if
	}
	//#next point
	if(min_dr_p != NULL)
	{
		cout << "Setting color!" << endl;
		min_dr_p->setColor("#cc0000");
		if(selectedPoint != NULL)
		{
			selectedPoint->setColor("#0000ff");
		}
		//#end if
		selectedPoint = min_dr_p;
	}
	//#end if
}
//#end void click(x, y)

void Map::sort1()
{
	uint32 i;
	for(i = 0; i < points.size(); i++)
	{
		uint32 j;
		for(j = 0; j < points[i].conns.size(); j++)
		{
			Vector2 dr = *(points[i].conns[j]) - points[i];
			//#(dx, dy) = (conn.x - point.x, conn.y - point.y)
			//points[i] += dr * 0.4;
			points[i].x += dr.x * 0.4;
			points[i].y += dr.y * 0.4;
		}
		//#next conn
	}
	//#next point
}
//#end void sort1()

// dist1 = 70, f = 0.5
void Map::sort2(float dist1, float f)
{
	/*
	Use springs to decide the position of each point.
	*/
	uint32 i;
	for(i = 0; i < points.size(); i++)
	{
		uint32 j;
		for(j = 0; j < points[i].conns.size(); j++)
		{
			//#(dx, dy) = (conn.x - point.x, conn.y - point.y)
			Vector2 dr = *(points[i].conns[j]) - points[i];
			//#dr_len = sqrt(dx*dx + dy*dy)
			float dr_len = dr.length();
			//#print '	dr:', dr
			if(dr_len == 0.0)
			{
				points[i].x += 5;
				points[i].y += 5;
				dr_len = sqrt(5*5 + 5*5);
				//#print 'zero dr is now:', dr
			}
			//#end if
			//#print 'non-zero dr:', dr
			float m = 1;
			float a_len = (f/m) * (dr_len - dist1);
			//#dr_hat = (dx/dr, dy/dr)
			Vector2 dr_hat = dr / dr_len;
			//#a = (a_len*dr_hat[0], a_len*dr_hat[1]/m)
			Vector2 a = dr_hat * (a_len / m);
			float dt = 0.6;
			//#dp = (0.5*dt*dt*a[0], 0.5*dt*dt*a[1])
			Vector2 dp = a * (0.5*dt*dt);
			points[i].x += dp.x;
			points[i].y += dp.y;
		}
		//#next conn
	}
	//#next point
}
//#end void sort2()

void Map::sort3()
{
	/*
	Use springs to decide the position of each point.
	*/
	uint32 i;
	for(i = 0; i < points.size(); i++)
	{
		if(points[i].conns.size() <= 1) continue;
		//#avgx = 0
		//#avgy = 0
		Vector2 avg = Vector2(0, 0);
		uint32 j;
		for(j = 0; j < points[i].conns.size(); j++)
		{
			//#avgx += conn.x
			//#avgy += conn.y
			avg += *(points[i].conns[j]);
		}
		//#next conn
		//#point.x = avgx / len(point.conns)
		//#point.y = avgy / len(point.conns)
		Vector2 p = avg / points[i].conns.size();
		points[i].x = p.x;
		points[i].y = p.y;
	}
	//#next point
}
//#end void sort3()

void Map::sort4()
{
	/*
	Use a style of reverse attack on non-conns that are close.
	*/
	uint32 i;
	for(i = 0; i < points.size(); i++)
	{
		uint32 j;
		for(j = 0; j < points.size(); j++)
		{
			// nonconn
			//if(points[j] in point.conns)
			Points::iterator it = points.end(); //find(points.begin(), points.end(), points[j]);
			if(it != points.end())
			{
				continue;
			}
			//#end if
			Vector2 dr = points[i] - points[j];
			float dr_len = dr.length();
			//#print 'dr_len =', dr_len
			if(dr_len < 50)
			{
				if(dr_len == 0)
				{
					dr = Vector2(1, 1);
					dr_len = sqrt(2);
				}
				//#end if
				dr *= 4.0 / dr_len;
				points[i].x += dr.x;
				points[i].y += dr.y;
			}
			//#end if
		}
		//#next nonconn
	}
	//#next point
}
//#end void sort4()

void Map::center()
{
	Vector2 mean = Vector2(0,0);
	uint32 i;
	for(i = 0; i < points.size(); i++)
	{
		//#mean[0] += point.x
		//#mean[1] += point.y
		mean += points[i];
	}
	//#next point
	//#mean[0] /= (1.0 * len(self.points))
	//#mean[1] /= (1.0 * len(self.points))
	mean /= (1.0 * points.size());
	//#goto = [self.width/2.0, self.height/2.0]
	Vector2 goto_pos = Vector2((int)(width/2.0), (int)(height/2.0));
	//#diff = [goto[0]-mean[0], goto[1]-mean[1]]
	Vector2 diff = goto_pos - mean;
	//#print 'center diff', diff
	for(i = 0; i < points.size(); i++)
	{
		//#point.x += diff[0]
		//#point.y += diff[1]
		points[i].x += diff.x;
		points[i].y += diff.y;
	}
	//#next point
}
//#end void center()

void Map::squash()
{
	Vector2 mean = Vector2(0,0);
	uint32 i;
	for(i = 0; i < points.size(); i++)
	{
		//#mean[0] += point.x
		//#mean[1] += point.y
		mean += points[i];
	}
	//#next point
	//#mean[0] /= (1.0 * len(self.points))
	//#mean[1] /= (1.0 * len(self.points))
	mean /= (1.0 * points.size());
	//#goto = [self.width/2.0, self.height/2.0]
	Vector2 goto_pos = Vector2((int)(width/2.0), (int)(height/2.0));
	//#diff = [goto[0]-mean[0], goto[1]-mean[1]]
	Vector2 diff = goto_pos - mean;
	for(i = 0; i < points.size(); i++)
	{
		//#point.x += diff[0]
		//#point.y += diff[1]
		// Note that this is a bug in center, but I'm leaving it in as squash.
		uint32 j;
		for(j = 0; j < points[i].conns.size(); j++)
		{
			points[i].x += diff.x;
			points[i].y += diff.y;
		}
		//#next conn
	}
	//#next point
}
//#end void squash()


string Map::str()
{
	return drawSvg();
}
//#void __str__(self):

string itostr(int a)
{
	string o;
	stringstream s;
	s << a;
	s >> o;
	return o;
}

string ftostr(float a)
{
	string o;
	stringstream s;
	s << a;
	s >> o;
	return o;
}

string Map::drawSvg()
{
	string output = "<?xml version=\"1.0\" standalone=\"no\"?>\n"
		"<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">";
	output += "\n<svg width=\"" + itostr(width) + "\" height=\"" + itostr(height) + "\" viewBox=\"0 0 " + itostr(width) + " " + itostr(height) + "\" xmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\">\n";
	output += "<title>Network Map 4 svg</title>\n";
	output += "<desc>Test of drawing a network map with svg</desc>\n";
	output += "<rect x=\"1\" y=\"1\" width=\"" + itostr(width-2) + "\" height=\"" + itostr(height-2) + "\" fill=\"none\" stroke=\"blue\" />\n";
	
	uint32 i;
	for(i = 0; i < points.size(); i++)
	{
		output += points[i].drawSvg(line_width);
	}
	//#next point
	//# A layer of text above the connections.
	output += "<g>\n";
	for(i = 0; i < points.size(); i++)
	{
		Vector2 tpos = points[i] + Vector2(8, 8);
		output += "<text x='" + itostr(tpos.x) + "' y='" + itostr(tpos.y) + "'>" + points[i].name + "</text>\n";
		//output += "<text x='" + ('%3.3f' % tpos.x) + "' y='" + ('%3.3f' % tpos.y) + "'>" + point.name + "</text>\n";
	}
	//#next point
	output += "</g>\n";
	output += "</svg>\n";
	return output;
}
// #end void drawSvg()

// #end class Map

//class Vector2:
// x=380, y=180
Vector2::Vector2(int init_x, int init_y)
{
	x = init_x;
	y = init_y;
}

Vector2::Vector2(Point *p)
{
	x = p->x;
	y = p->y;
}

Vector2::Vector2()
{
	x = 0;
	y = 0;
}

Vector2::~Vector2()
{
}

string Vector2::str()
{
	string s;
	//('%3.3f' % self.x) + ', ' + ('%3.3f' % self.y);
	s = itostr(x) + ", " + itostr(y);
	return s;
}
//#end void __str__()

Vector2::operator string ()
{
	return str();
}

Vector2 Vector2::operator +(Vector2 v)
{
	return Vector2(x + v.x, y + v.y);
}

Vector2 Vector2::operator -(Vector2 v)
{
	return Vector2(x - v.x, y - v.y);
}

Vector2 Vector2::operator *(float f)
{
	return Vector2((int)(x * f), (int)(y * f));
}

Vector2 Vector2::operator /(float f)
{
	return Vector2((int)(x / f), (int)(y / f));
}

Vector2 &Vector2::operator +=(Vector2 v)
{
	x += v.x;
	y += v.y;
	return *this;
}

Vector2 &Vector2::operator -=(Vector2 v)
{
	x -= v.x;
	y -= v.y;
	return *this;
}

Vector2 &Vector2::operator *=(float f)
{
	x *= f;
	y *= f;
	return *this;
}

Vector2 &Vector2::operator /=(float f)
{
	x /= f;
	y /= f;
	return *this;
}

//#void __iadd__(vec):
//#	v = Vector2(self.x + vec.x, self.y + vec.y)
//#	return v

//#void __idiv__(flt):
//#	v = Vector2(self.x / flt, self.y / flt)
//#	return v

float Vector2::length()
{
	return sqrt(x * x + y * y);
}

//#end class Vector2

//class Point(Vector2):
//name, conns=None, x=380, y=180, color='//#0000ff'
Point::Point(string init_name, int init_x, int init_y, Pointps *init_conns, string init_color)
: Vector2(init_x, init_y)
{
	//Vector2::Vector2(init_x, init_y);
	name = init_name;
	if(init_conns != NULL)
	{
		conns = *init_conns;
	}
	color = init_color;
	//if conns == None: self.conns = []
}

Point::~Point()
{
}

string Point::str()
{
	string s;
	//cout << name << endl;
	s = name + " " + itostr(x) + ", " + itostr(y); //Vector2::str();
	return s;
}
//#end void __str__()

string Point::drawSvg(int line_width)
{
	string output;
	//Vector2 tpos = *this + Vector2(8, 8);
	//# Draw each point
	//output = "<!--<text x='" + itostr(tpos.x) + "' y='" + itostr(tpos.y) + "'>" + name + "</text>-->\n";
	output += "<circle cx='" + itostr(x) + "' cy='" + itostr(y) + "' r='4' fill='" + color + "' stroke='black' stroke-width='1' />\n";
	//# Draw each conn
	uint32 j;
	for(j = 0; j < conns.size(); j++)
	{
		output += "<!-- " + this->str() + " to " + conns[j]->str() + "-->\n";
		output += "<path d='M " + itostr(x) + " " + itostr(y) + " L " + itostr(conns[j]->x) + " " + itostr(conns[j]->y) + "' fill='none' stroke='blue' stroke-width='" + itostr(line_width) + "' />\n";
	}
	//#next conn
	return output;
}

// conn, reConn=True
void Point::addConn(Point *conn, bool reConn)
{
	//#print 'adding conn?', self.name, conn
	conns.push_back(conn);
	if(reConn)
	{
		conn->addConn(this, false);
	}
}

void Point::setColor(string value)
{
	// Set color to a hex value like //#ff0000 //
	color = value;
}
//#end class Point(Vector2)

#ifdef TEST_SWW

int main(int argc, char **argv)
{
	cout << "Vector2" << endl;
	Vector2 v;
	cout << v.str() << endl;
	v = Vector2(62, 99);
	cout << v.str() << endl;
	v += Vector2(22, 11);
	cout << v.str() << endl;

	cout << "Point + Vector2" << endl;
	Point p("p", 0, 0, NULL, "#000000");
	cout << p.str() << endl;
	cout << v.str() << endl;
	p += v;
	cout << p.str() << endl;
	cout << v.str() << endl;
	
	cout << "Points" << endl;
	Points s;
	s.push_back(Point("a", 123, 456));
	s.push_back(Point("b", 789, 234));
	s.push_back(Point("c", 567, 890));
	cout << s[0].str() << endl;
	cout << s[1].str() << endl;
	cout << s[2].str() << endl;

	s[0] += v;
	cout << s[0].str() << endl;

	return 0;
}


#endif // TEST_SWW
