#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
cppmap1.py
by Javantea
Sept 5, 2009

This module creates variable descriptions from cpplang2 output.

"""
from __future__ import print_function
import re
from random import seed
from sys import path
path.append('.')
import mapmake1

def mapCppLang(filename):
	"""
	mapDisasm takes a dump from cpplang2.py and turns it into a useful 
	list of functions, vars and the relationships that they have.
	"""
	a = open(filename, 'r').read()
	b = a.split('\n')
	functions = []
	variables = []
	current_global_vars = True
	current_functions = False
	current_undeclared_vars = False
	var_in_progress = None
	func_in_progress = None
	for line in b:
		if current_global_vars and (line == 'Global Vars:' or line == '/*'):
			#print 'cgv', functions, variables
			current_global_vars = True
			continue
		#end if
		if current_global_vars and (line == 'Functions:'):
			#print 'fcn', functions, variables
			if var_in_progress != None:
				variables.append(var_in_progress)
				var_in_progress = None
			#end if
			current_functions = True
			#print '/fcn', functions, variables
			continue
		#end if
		if current_functions and (line == 'Undeclared Vars:'):
			#print 'udv', functions, variables
			if var_in_progress != None:
				# Put the previous one onto the stack.
				if func_in_progress != None:
					func_in_progress[1].append(var_in_progress)
				else:
					variables.append(var_in_progress)
				#end if
				var_in_progress = None
			#end if
			if func_in_progress != None:
				# Put the previous one onto the stack.
				functions.append(func_in_progress)
				func_in_progress = None
			#end if
			current_undeclared_vars = True
			#print '/udv', functions, variables
			continue
		#end if
		if current_undeclared_vars and (line == '*/'):
			#print 'gb', functions, variables
			continue
		#end if
		if len(line) == 0:
			# Blank line
			continue
		#end if
		
		if line[0] == '[':
			# It's a variable.
			if var_in_progress != None:
				# Put the previous one onto the stack.
				if func_in_progress != None:
					func_in_progress[1].append(var_in_progress)
				else:
					variables.append(var_in_progress)
				#end if
			#end if
			var_in_progress = (getVarData(line), [])
		elif line[0] == '\t' or line[0] == ' ':
			var_in_progress[1].append(line.strip())
		elif current_functions:
			# Functions can only be declared in functions.
			if func_in_progress != None:
				# Put the previous one onto the stack.
				functions.append(func_in_progress)
			#end if
			func_in_progress = (line, [])
		else:
			# The line is unparsable?
			print("Unparsable:", line)
		#end if
	#next line
	
	if var_in_progress != None:
		# Put the previous one onto the stack.
		if func_in_progress != None:
			func_in_progress[1].append(var_in_progress)
		else:
			variables.append(var_in_progress)
		#end if
	#end if
	if func_in_progress != None:
		# Put the previous one onto the stack.
		functions.append(func_in_progress)
		func_in_progress = None
	#end if
	
	return functions, variables
#end def mapCppLang(filename)

def getVarData(var_def):
	var_items = var_def.split(' ')
	nextVar = False
	varname = ''
	for item in var_items:
		if nextVar:
			varname = item
			nextVar = False
		elif item == '[var]':
			nextVar = True
		#end if
	#next item
	return varname
#end def getVarData(var_def)

def cppmap(filename='cppfor3.cl2', seed_id = 2345):
	seed(seed_id)
	m = mapmake1.Map()
	#'/home/jvoss/recent/pfm/projects/objmap/python1.dis'
	f, v = mapCppLang(filename)
	
	"""# the format of functions is:
	# [func name, (var name, var name, ...)]
	for func in f:
		print func[0] + ': (',
		for var in func[1]:
			print var + ',',
		#next var
		print ')'
	#next func
	return"""
	
	# The variable callersOnly filters out those functions that make no calls.
	# The graph removes dots which are less interesting and far more common in 
	# crazy binaries. The map will still show dots where the caller's called 
	# item has been removed. TODO: write code to filter only caller/called.
	
	i = 0
	p = []
	names = {}
	max_func = ''
	max_vars = 0
	for func in f:
		p.append(mapmake1.Point(func[0]))
		names[func[0]] = i
		i += 1
		if len(func[1]) > max_vars:
			max_vars = len(func[1])
			max_func = func[0]
		#end if
	#next func
	i = 0
	for func in f:
		for var in func[1]:
			if var[0] in names:
				conn = names[var[0]]
				p[i].addConn(p[conn])
			else:
				#print "Error: names doesn't contain", var
				j = len(p)
				p.append(mapmake1.Point(var[0]))
				p[i].addConn(p[j])
				names[var[0]] = j
			#end if
		#next call
		i += 1
	#next func
	
	m.addPoints(p)
	for i in range(400):
		m.randomize(15)
		m.sort2()
		#a.sort1()
		#a.sort1()
		#a.sort1()
		#a.sort1()
	#next i
	
	m.center()
	
	m.max_func = max_func
	m.max_vars = max_vars
	
	return m
#end def objmap(seed_id)

def main(seed_id = 2345):
	filename = 'data/cppfor3.cl2'
	#callersOnly = False
	
	from sys import argv
	#if '-c' in argv:
	#	argv.pop(argv.index('-c'))
	#	callersOnly = True
	#	print argv
	#end if
	if '-m' in argv:
		import pprint
		argv.pop(argv.index('-m'))
		f, v = mapCppLang(filename)
		pprint.pprint(f)
		pprint.pprint(v)
		return
	#end if
	if len(argv) > 1:
		filename = argv[1]
	#end if
	
	m = cppmap(filename, seed_id)
	print(m)
	print("<!-- max_func =", m.max_func, ", max_vars =", m.max_vars, " -->")
#end def main(seed_id)

if __name__ == '__main__':
	main()
#end if



