#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
cppmap1.py
by Javantea
Sept 5, 2009

This module creates variable descriptions from cpplang2 output.

"""
from sys import path, stderr
path.append('.')
import mapmake1
import re
from random import seed
import cpplang2


def mapCppLang(filename, var_select=None):
	"""
	mapDisasm takes a dump from cpplang2.py and turns it into a useful 
	list of functions, vars and the relationships that they have.
	"""
	cpplang2.verbose_print = cpplang2.non_verbose_print
	cpplang2.vverbose_print = cpplang2.non_verbose_print
	cpplang2.vvverbose_print = cpplang2.non_verbose_print
	level1 = cpplang2.parseFile(filename)
	level2 = cpplang2.lev2parser(level1)
	
	functions = []
	variables = []
	funcs = cpplang2.getFuncs(level2)
	for func in funcs:
		"""
		bufs = cpplang2.compareBufIndices(func)
		lvars = []
		for (varname, e) in bufs.items():
			calls = e[0] + e[1]
			if len(e[0]) > 0 and type(e[0][0]) == list:
				#print 'z', e[0]
				calls = e[0][0] + e[1]
			#end if
			#print 'a', varname, calls
			rels = getRelationship(calls)
			#print 'b', rels
			lvars.append((varname, rels))
			
			# Get another level
			for relvar in rels:
				calls, w = cpplang2.findVar(func, varname)
				if calls == True:
					calls = [w]
				#end if
				rels2 = set(getRelationship(calls))
				lvars.append((relvar, rels2))
			#end if
		#next varname, e
		"""
		if var_select != None:
			local_vars = [var_select]
		else:
			local_vars = cpplang2.getLocalVars(func, True)
			if len(local_vars) == 0:
				print('rather unlikely eh?')
				continue
			#end if
		#end if
		#lvars = [str(a) for a in local_vars]
		lvars = []
		for v in local_vars:
			if type(v) == str:
				varname = v
			else:
				var_list = getRelationship([v])
				print('vl', var_list)
				if len(var_list) == 0:
					if len(local_vars) == 0:
						print('unlikely again eh?')
						continue
					#end if
					continue
				#end if
				varname = var_list[0]
			#end if
			calls, w = cpplang2.findVar(func, varname)
			#print 'v', v
			if calls == True:
				calls = [w]
			#end if
			rels = set(getRelationship(calls))
			lvars.append((varname, rels))
			
			# Get another level
			for relvar in rels:
				calls, w = cpplang2.findVar(func, varname)
				if calls == True:
					calls = [w]
				#end if
				rels2 = set(getRelationship(calls))
				lvars.append((relvar, rels2))
			#end if
			#print 'r', rels
				#next i
			#next val
		#next v
		if var_select != None and lvars == []:
			continue
		#end if
		#"""
		functions.append((func.definition(), lvars))
	#next line
	
	#print 'f', functions
	
	return functions, variables
#end def mapCppLang(filename)

def getRelationship(c):
	"""
	A bunch of statements that contain a similar var.
	"""
	output = []
	for statement in c:
		if statement.__class__ == cpplang2.CVar:
			output.append(statement.value)
			continue
		#end if
		c_set = False
		try:
			raise statement
		except cpplang2.CSet:
			c_set = True
		except:
			pass
		#end try
		look = [statement.value]
		if c_set:
			look.append(statement.lhs)
		#end if
		#print 'l', look
		for val in look:
			if type(val) == list:
				output.extend(getRelationship(val))
			#end if
		#next val
	#next statement
	
	return output
#end if getRelationship(calls)

def cppmap(filename='cppfor3.cl2', var=None, seed_id = 2345):
	seed(seed_id)
	width = 800
	height = 600
	m = mapmake1.Map(None, width, height, 1, '#9999ff')
	#'/home/jvoss/recent/pfm/projects/objmap/python1.dis'
	f, v = mapCppLang(filename, var)
	
	"""# the format of functions is:
	# [func name, (var name, var name, ...)]
	for func in f:
		print func[0] + ': (',
		for var in func[1]:
			print var + ',',
		#next var
		print ')'
	#next func
	return"""
	
	# The variable callersOnly filters out those functions that make no calls.
	# The graph removes dots which are less interesting and far more common in 
	# crazy binaries. The map will still show dots where the caller's called 
	# item has been removed. TODO: write code to filter only caller/called.
	
	i = 0
	p = []
	names = {}
	max_func = ''
	max_vars = 0
	for func in f:
		p.append(mapmake1.Point(func[0], color='#ff9966'))
		names[func[0]] = i
		i += 1
		if len(func[1]) > max_vars:
			max_vars = len(func[1])
			max_func = func[0]
		#end if
	#next func
	i = 0
	for func in f:
		for var in func[1]:
			if var[0] in names:
				conn = names[var[0]]
				p[i].addConn(p[conn])
			else:
				#print "Error: names doesn't contain", var
				j = len(p)
				p.append(mapmake1.Point(var[0], color='#99ccff'))
				p[i].addConn(p[j])
				names[var[0]] = j
			#end if
			#print 're', var[1]
			for rel in var[1]:
				if rel in names:
					conn = names[rel]
					p[names[var[0]]].addConn(p[conn])
				else:
					#print "Error: names doesn't contain", var
					j = len(p)
					p.append(mapmake1.Point(rel, color='#99ccff'))
					p[names[var[0]]].addConn(p[j])
					names[rel] = j
				#end if
			#next var
		#next var
		i += 1
	#next func
	
	m.addPoints(p)

	w = 100
	h = 100
	m.randomize(int((w + h) / 1.4))
	for i in range(40):
		m.sort2((w + h) / 15)
		if i % 30 == 0:
			m.sort3()
		#end if
		if i % 32 == 0:
			m.sort4()
		#end if
		if i % 25 == 0:
			m.randomize(120)
		#end if
		#a.sort1()
		#a.sort1()
		#a.sort1()
		#a.sort1()
	#next i
	for i in range(25):
		m.sort4()
	#next i
	#m.sort3()
	m.center()
	
	"""for i in range(300):
		m.randomize(40)
		m.sort2()
		if i % 20:
			m.sort4()
		#end if
		#a.sort1()
		#a.sort1()
		#a.sort1()
		#a.sort1()
	#next i
	
	m.center()
	"""
	m.max_func = max_func
	m.max_vars = max_vars
	
	return m
#end def objmap(seed_id)

def main(seed_id = 2345):
	filename = 'data/cppfor3.c'
	var = None
	#callersOnly = False
	
	from sys import argv
	#if '-c' in argv:
	#	argv.pop(argv.index('-c'))
	#	callersOnly = True
	#	print argv
	#end if
	if '-m' in argv:
		import pprint
		argv.pop(argv.index('-m'))
		f, v = mapCppLang(filename)
		pprint.pprint(f)
		pprint.pprint(v)
		return
	#end if
	if len(argv) > 1:
		filename = argv[1]
	#end if
	if len(argv) > 2:
		var = argv[2]
	#end if
	
	m = cppmap(filename, var, seed_id)
	print(m)
	print("<!-- max_func =", m.max_func, ", max_vars =", m.max_vars, " -->")
#end def main(seed_id)

if __name__ == '__main__':
	main()
#end if



