#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
cppmap1.py
by Javantea
Sept 5, 2009

This module creates variable descriptions from cpplang2 output.

"""
from sys import path, stderr
path.append('.')
import mapmake1
import re
from random import seed
import cpplang2


def mapCppLang(filename, var_select=None):
	"""
	mapDisasm takes a dump from cpplang2.py and turns it into a useful 
	list of functions, vars and the relationships that they have.
	"""
	cpplang2.verbose_print = cpplang2.non_verbose_print
	cpplang2.vverbose_print = cpplang2.non_verbose_print
	cpplang2.vvverbose_print = cpplang2.non_verbose_print
	level1 = cpplang2.parseFile(filename)
	level2 = cpplang2.lev2parser(level1)
	
	cpplang2.printRelationships = True
	
	printStuff = False

	solver = cpplang2.CAdvancedSolver(level2)

	functions = []
	variables = []
	
	for func in solver.funcs:
		lvars = []
		funcname = func.funcname
		
		funcCalledBy = []
		if funcname in solver.calledBy:
			funcCalledBy = solver.calledBy[funcname]
		#end if

		vetted = ''
		# Step 5.a: print out the function definition.
		if printStuff: print(cpplang2.colorText('[f]', 'red'), func.definition())
		
		#if not veryObvious:
		#	printVars(solver.local_vars[funcname])
		#end if
		
		# Step 5.c: Find buffer
		bufDict, riskyFors = cpplang2.findBufferUses(func, solver)
		unsafeFors, safeFors, otherBufSets = cpplang2.filterBufFors(func, riskyFors, solver)
		if unsafeFors:
			vetted = 'fors, '
			if printStuff:
				print(cpplang2.colorText("For Loop Buffer Sets:", 'blue'))
				cpplang2.printUnsafeFors(unsafeFors, func, solver)
		#end if
		if printStuff: print(cpplang2.colorText("[#AF]", 'orange'), len(safeFors))
		#if len(safeFors) != 0 or not veryObvious:
		#	printSafeFors(safeFors)
		#end if
		if bufDict != {}:
			riskyDict, safeDict = cpplang2.filterBufSets(bufDict)
			if riskyDict != {}:
				vetted += 'buf ind, '
				#rels = set(getRelationship(calls))
				rels = []
				for (varname, vals) in list(riskyDict.items()):
					print(vals)
					# Each of these are allocs.
					rels = []
					if vals[0] and vals[0].allocs and vals[0].allocs[0]:
						rels = [str(vals[0].allocs[0][0].value)]
					#end if
					lvars.append((varname, rels))
				#next varname
			#end if
			
			if printStuff:
				print(cpplang2.colorText("Buffers:", 'blue'))
				cpplang2.printCompareBuf(riskyDict)
			#print
		#end if
		
		#Step 5.d: Find dereferences
		riskyDerefs = cpplang2.findRiskyDerefs(func, solver)
		if printStuff: cpplang2.printDerefs(riskyDerefs)
		if riskyDerefs != []:
			vetted += 'derefs, '
			for var in riskyDerefs:
				# Each of these are allocs.
				rels = []
				#if vals[0] and vals[0][0] and vals[0][0][0]:
				#	rels = [str(vals[0][0][0][0].value)]
				#end if
				print(var[0])
				lvars.append((str(var[0]), rels))
			#next varname
		#end if
		
		#Step 5.e: Find calls
		#if not veryObvious:
		#	print cpplang2.colorText("Calls:", 'blue')
		#	calls = findCalls(func)
		#	printCalls2(calls)
		#end if
		
		#Step 5.f: Find function return values
		"""if not veryObvious:
			print cpplang2.colorText("Function Return Value:", 'blue')
			returned_value = maxReturn(func)
			if type(returned_value) != list:
				print '\tMax: [r]', returned_value
			else:
				if len(returned_value) == 0:
					print "\t[r] does not return a value"
				else:
					for item in returned_value:
						print '\tMax: [r]', item #' '.join([str(s) for s in item])
					#next item
				#end if
			#end if
			
			if type(returned_value) != list or len(returned_value) > 0:
				returned_value = minReturn(func)
				if type(returned_value) != list:
					print '\tMin: [r]', returned_value
				else:
					if len(returned_value) == 0:
						print "\tMin: [r] does not return a value"
					else:
						for item in returned_value:
							print '\tMin: [r]', item #' '.join([str(s) for s in item])
						#next item
					#end if
				#end if
			#end if
		#end if"""
		
		# Step 5.g: Find buffer functions and print their calls
		bufferCalls = cpplang2.findBufferFunctions(func, solver)
		if bufferCalls:
			if printStuff:
				print(cpplang2.colorText("Buffer Functions:", 'blue'))
				cpplang2.printCalls2(bufferCalls)
			vetted += 'buffer funcs, '
		#end if
		
		if vetted == '':
			vetted = 'vetted'
		else:
			vetted = vetted[:-2]
		#end if
		if printStuff: print(cpplang2.colorText("Vetting:", 'blue'), vetted)
			
		if lvars != []:
			functions.append((func.definition(), lvars))
		#end if
	#next func
	
	#print 'f', functions
	
	return functions, variables
#end def mapCppLang(filename)

def getRelationship(c):
	"""
	A bunch of statements that contain a similar var.
	"""
	output = []
	for statement in c:
		if statement.__class__ == cpplang2.CVar:
			output.append(statement.value)
			continue
		#end if
		c_set = False
		try:
			raise statement
		except cpplang2.CSet:
			c_set = True
		except:
			pass
		#end try
		look = [statement.value]
		if c_set:
			look.append(statement.lhs)
		#end if
		#print 'l', look
		for val in look:
			if type(val) == list:
				output.extend(getRelationship(val))
			#end if
		#next val
	#next statement
	
	return output
#end if getRelationship(calls)

def cppmap(filename='cppfor3.cl2', var=None, seed_id = 2345):
	seed(seed_id)
	width = 800
	height = 600
	m = mapmake1.Map(None, width, height, 1, '#9999ff')
	#'/home/jvoss/recent/pfm/projects/objmap/python1.dis'
	f, v = mapCppLang(filename, var)
	
	"""# the format of functions is:
	# [func name, (var name, var name, ...)]
	for func in f:
		print func[0] + ': (',
		for var in func[1]:
			print var + ',',
		#next var
		print ')'
	#next func
	return"""
	
	# The variable callersOnly filters out those functions that make no calls.
	# The graph removes dots which are less interesting and far more common in 
	# crazy binaries. The map will still show dots where the caller's called 
	# item has been removed. TODO: write code to filter only caller/called.
	
	i = 0
	p = []
	names = {}
	max_func = ''
	max_vars = 0
	for func in f:
		p.append(mapmake1.Point(func[0], color='#ff9966'))
		names[func[0]] = i
		i += 1
		if len(func[1]) > max_vars:
			max_vars = len(func[1])
			max_func = func[0]
		#end if
	#next func
	i = 0
	for func in f:
		for var in func[1]:
			if var[0] in names:
				conn = names[var[0]]
				p[i].addConn(p[conn])
			else:
				#print "Error: names doesn't contain", var
				j = len(p)
				p.append(mapmake1.Point(var[0], color='#99ccff'))
				p[i].addConn(p[j])
				names[var[0]] = j
			#end if
			#print 're', var[1]
			for rel in var[1]:
				if rel in names:
					conn = names[rel]
					p[names[var[0]]].addConn(p[conn])
				else:
					#print "Error: names doesn't contain", var
					j = len(p)
					p.append(mapmake1.Point(rel, color='#99ccff'))
					p[names[var[0]]].addConn(p[j])
					names[rel] = j
				#end if
			#next var
		#next var
		i += 1
	#next func
	
	m.addPoints(p)

	w = 300
	h = 300
	m.randomize(int((w + h) / 1.4))
	for i in range(40):
		m.sort2((w + h) / 15)
		if i % 30 == 0:
			m.sort3()
		#end if
		if i % 32 == 0:
			m.sort4()
		#end if
		if i % 25 == 0:
			m.randomize(120)
		#end if
		#a.sort1()
		#a.sort1()
		#a.sort1()
		#a.sort1()
	#next i
	for i in range(25):
		m.sort4()
	#next i
	#m.sort3()
	m.center()
	
	"""for i in range(300):
		m.randomize(40)
		m.sort2()
		if i % 20:
			m.sort4()
		#end if
		#a.sort1()
		#a.sort1()
		#a.sort1()
		#a.sort1()
	#next i
	
	m.center()
	"""
	m.max_func = max_func
	m.max_vars = max_vars
	
	return m
#end def objmap(seed_id)

def main(seed_id = 2345):
	filename = 'data/cppfor3.c'
	var = None
	#callersOnly = False
	
	from sys import argv
	#if '-c' in argv:
	#	argv.pop(argv.index('-c'))
	#	callersOnly = True
	#	print argv
	#end if
	if '-m' in argv:
		import pprint
		argv.pop(argv.index('-m'))
		f, v = mapCppLang(filename)
		pprint.pprint(f)
		pprint.pprint(v)
		return
	#end if
	if len(argv) > 1:
		filename = argv[1]
	#end if
	if len(argv) > 2:
		var = argv[2]
	#end if
	
	m = cppmap(filename, var, seed_id)
	print(m)
	print("<!-- max_func =", m.max_func, ", max_vars =", m.max_vars, " -->")
#end def main(seed_id)

if __name__ == '__main__':
	main()
#end if



