#!/usr/bin/env python3
"""
Small Wide World
Map maker
by Javantea
April 26, 2007

Description:
An idea to make maps.
1: Place all points at the center.
2: Move each point randomly.
3: Move each point closer to it's connected points.
4: Goto 2
5: Profit!

TODO:
Get the randomization.
Get a data set to draw.
Get the flocking.
Output SVG.
"""
from __future__ import print_function
from random import randint, seed
from math import sqrt, sin, cos, pi, atan2
tau = 2 * pi

class Map:
	"""
	Base class for all maps (friend, object, route).
	Write the hard math and physics here once and use it by calling once.
	"""
	def __init__(self, points=None, width=640, height=480, line_width=2, line_color = '#8888ff'):
		self.points = points
		# There's a bug in python where if I do points=[] above, it 
		# thinks it's a static when it's really really not.
		if points == None: self.points = []
		self.width  = width
		self.height = height
		self.line_width = line_width
		self.line_color = line_color
		self.selectedPoint = None
		self.title = 'Small Wide World Map'
		self.desc = 'Test of drawing a network map with svg'

	def addPoint(self, point):
		self.points.append(point)

	def addPoints(self, points):
		self.points.extend(points)
	
	def randomize(self, amount = 180):
		for point in self.points:
			point.x += randint(-amount, amount)
			point.y += randint(-amount, amount)
		#next point
	#end def randomize()
	
	def drag(self, x, y):
		if self.selectedPoint == None:
			return False
		#end if
		r = Vector2(x, y)
		d_len = (r - self.selectedPoint).length()
		if (d_len > 10):
			self.selectedPoint.x = x
			self.selectedPoint.y = y
			return True
		#end if
		return False
	#end def drag(x, y)
	
	def click(self, x, y):
		cp = Vector2(x, y)
		min_dr = -1.0
		min_dr_p = None
		for point in self.points:
			dr = cp - point
			dr_len = dr.length()
			if (min_dr < 0) or (dr_len < min_dr):
				min_dr = dr_len
				min_dr_p = point
			#end if
		#next point
		if min_dr_p != None:
			print("Setting color!")
			min_dr_p.setColor('#cc6666')
			if self.selectedPoint != None:
				self.selectedPoint.setColor('#6666ff')
			#end if
			self.selectedPoint = min_dr_p
		#end if
	#end def click(x, y)
	
	def sort1(self):
		for point in self.points:
			for conn in point.conns:
				dr = conn - point
				#(dx, dy) = (conn.x - point.x, conn.y - point.y)
				point += dr * 0.4
				#point.x += dx * 0.4
				#point.y += dy * 0.4
			#next conn
		#next point
	#end def sort1()
	
	def sort2(self, dist1 = 80, f = 0.5):
		"""
		Use springs to decide the position of each point.
		"""
		for point in self.points:
			for conn in point.conns:
				#(dx, dy) = (conn.x - point.x, conn.y - point.y)
				dr = conn - point
				#dr_len = sqrt(dx*dx + dy*dy)
				dr_len = dr.length()
				#print '	dr:', dr
				if dr_len == 0.0:
					point.x += 5.0
					point.y += 5.0
					dr_len = sqrt(5*5 + 5*5)
					#print 'zero dr is now:', dr
				#end if
				#print 'non-zero dr:', dr
				m = 1
				a_len = (f/m) * (dr_len - dist1)
				#dr_hat = (dx/dr, dy/dr)
				dr_hat = dr / dr_len
				#a = (a_len*dr_hat[0], a_len*dr_hat[1]/m)
				a = dr_hat * (a_len / m)
				dt = 0.6
				#dp = (0.5*dt*dt*a[0], 0.5*dt*dt*a[1])
				dp = a * (0.5*dt*dt)
				point.x += dp.x
				point.y += dp.y
			#next conn
		#next point
	#end def sort2()
	
	def sort3(self):
		"""
		Use springs to decide the position of each point.
		"""
		for point in self.points:
			if len(point.conns) <= 1: continue
			#avgx = 0
			#avgy = 0
			avg = Vector2(0, 0)
			for conn in point.conns:
				#avgx += conn.x
				#avgy += conn.y
				avg += conn
			#next conn
			#point.x = avgx / len(point.conns)
			#point.y = avgy / len(point.conns)
			p = avg / len(point.conns)
			point.x = p.x
			point.y = p.y
		#next point
	#end def sort3()
	
	def sort4(self):
		"""
		Use a style of reverse attack on non-conns that are close.
		"""
		for point in self.points:
			for nonconn in self.points:
				if nonconn in point.conns:
					continue
				#end if
				dr = point - nonconn
				dr_len = dr.length()
				#print 'dr_len =', dr_len
				if dr_len < 50:
					if dr_len == 0:
						dr = Vector2(1, 1)
						dr_len = sqrt(2)
					#end if
					dr *= 4.0 / dr_len
					point.x += dr.x
					point.y += dr.y
				#end if
			#next nonconn
		#next point
	#end def sort4()
	
	def sort5(self):
		"""
		This is obviously the best algorithm ever. Hooray for the power 
		of awesomeness.
		"""
		for point in self.points:
			enemies_pos = Vector2(0, 0)
			enemies_len = 0
			friends_pos = Vector2(0, 0)
			for nonconn in self.points:
				if nonconn in point.conns:
					friends_pos += nonconn
					continue
				#end if
				dr = point - nonconn
				dr_len = dr.length()
				#print 'dr_len =', dr_len
				if dr_len < 50:
					enemies_pos += nonconn
					enemies_len += 1
				#end if
			#next nonconn
			enemies_pos /= enemies_len
			friends_pos /= len(point.conns)
			dr = friends_pos * 2 - enemies_pos
			# Our position looks like this: enemies ---- friends ---- me
			point.x = dr.x
			point.y = dr.y
			print(point)
			print('\t' + '(' + str(enemies_len) + ')', enemies_pos)
			print('\t' + '(' + str(len(point.conns)) + ')', friends_pos)
		#next point
	#end def sort5()
	
	def sort6(self):
		"""
		A new algorithm based on sort5 but with added validity information.
		"""
		validity = [False for p in self.points]
		for i, point in enumerate(self.points):
			enemies_pos = Vector2(0, 0)
			enemies_len = 0
			friends_pos = Vector2(0, 0)
			friends_len = 0
			for j, nonconn in enumerate(self.points):
				if not validity[j]:
					continue
				#end if
				if nonconn in point.conns:
					friends_pos += nonconn
					friends_len += 1
					continue
				#end if
				"""
				dr = point - nonconn
				dr_len = dr.length()
				print 'dr_len =', dr_len
				if dr_len < 150:
				"""
				enemies_pos += nonconn
				enemies_len += 1
				#end if
			#next nonconn
			if enemies_len > 1: enemies_pos /= enemies_len
			if friends_len > 1: friends_pos /= friends_len
			if enemies_len > 0 or friends_len > 0:
				dr = friends_pos * 1.7 - enemies_pos
				if enemies_len == 0 or (dr - friends_pos).length() < 1 or (dr - enemies_pos).length() < 1:
					# Fix anything by using a random angle.
					theta = randint(0, 360) * (pi / 180.0)
					dr = Vector2(dr.x + 30 * cos(theta), dr.y + 30 * sin(theta))
				#end if
			else:
				dr = Vector2(0, 0)
			#end if
			# Our position looks like this: enemies ---- friends ---- me
			point.x = dr.x + (randint(0, 100) / 4.0)
			point.y = dr.y + (randint(0, 100) / 4.0)
			#print point
			#print '\t' + '(' + str(enemies_len) + ')', enemies_pos
			#print '\t' + '(' + str(friends_len) + ')', friends_pos
			validity[i] = True
		#next point
	#end def sort6()
	
	def sortBranch(self, r0=40):
		"""
		A brilliant sorting method that works well with branches but not loops.
		r0 is the length of the bonds set by the algorithm.
		TODO: roundTry appears to be unnecessary (it always solve in one round).
		"""
		nodes = len(self.points)
		if nodes < 1:
			# Congrats, you're guaranteed to be sorted.
			return
		#end if
		# This is our algorithm... Written on the whiteboard last night. Understood many years ago but unable to write the code. Until now!
		knownPos = [False] * nodes
		knownPos[0] = True
		self.points[0].x = self.width / 2
		self.points[0].y = self.height / 2
		# v1 in spherical coordinates.
		#v1 = (r1, theta1) == (r1*cos(theta1), r1*sin(theta1))
		v1 = (r0, 0)
		# -v1 in spherical coordinates
		neg_v1 = (r0, v1[1] + pi)
		for roundTry in range(nodes):
			for i in range(nodes):
				#print(self.points[i].name, self.points[i].conns)
				givens = []
				unknowns = []
				# Set the position of its connections.
				for conn in self.points[i].conns:
					j = self.points.index(conn)
					if knownPos[j]:
						givens.append(j)
						continue
					#end if
					unknowns.append(j)
				#next conn
				if knownPos[i]:
					# First solve the easy cases, then work your way up.
					if len(unknowns) == 0:
						# Everything is known, we're good.
						#print('Yay', self.points[i].x, self.points[i].y)
						continue
					#end if
					if len(givens) == 0:
						# All unknowns, so use the simplex method.
						n_conns = len(unknowns)
						theta0 = tau / n_conns # tau = 2*pi
						x = 0
						for j in unknowns:
							# Spherical coordinates
							theta_j = v1[1] + (theta0 * (x+1))
							#v_j = (r0, theta_j)
							self.points[j].x, self.points[j].y = (self.points[i].x + (r0*cos(theta_j)), self.points[i].y + (r0*sin(theta_j)))
							knownPos[j] = True
							#print('Simple', x, j, self.points[j].x, self.points[j].y, theta_j)
							x += 1
						#next j
						continue
					#end if
					if len(givens) == 1:
						# Very easy, create a v1 and do the same as above
						v1i_cartesian = self.points[i] - self.points[givens[0]]
						v1i_theta = atan2(v1i_cartesian.y, v1i_cartesian.x)
						#v1i_spherical = (v1i_cartesian.length(), v1i_theta)
						#neg_v1i_spherical = (v1i_cartesian.length(), v1i_theta+pi)
						n_conns = len(unknowns) + 1
						theta0 = tau / n_conns # tau = 2*pi
						x = 0
						for j in unknowns:
							# Spherical coordinates
							if n_conns == 2:
								theta_j = v1i_theta
							else:
								theta_j = v1i_theta+pi + (theta0 * (x+1))
							#end if
							#v_j = (r0, theta_j)
							self.points[j].x, self.points[j].y = (self.points[i].x + (r0*cos(theta_j)), self.points[i].y + (r0*sin(theta_j)))
							knownPos[j] = True
							#print('Simple1', x, j, self.points[j].x, self.points[j].y, theta_j, 'v1it', v1i_theta)
							x += 1
						#next j
					#end if
					#print("Not sure what to do for: given: %i, unknown: %i" % (len(givens), len(unknowns)))
					# Take the list of givens and create a perfect placement for our node
				else:
					#print("Not known pos", i, knownPos[i])
					if len(givens) == 0:
						# We don't know any of our neighbors yet, nope.
						continue
					#end if
					if len(givens) == 1:
						# Use v1 to choose our position
						knownPos[i] = True
						j = givens[0]
						#print(i, 'using v_%i' % j)
						theta_j = v1[1]
						self.points[i].x, self.points[i].y = (self.points[j].x + (r0*cos(theta_j)), self.points[j].y + (r0*sin(theta_j)))
						continue
					#end if
					#print('avg')
					# Average of its peers
					totalPos = [0, 0]
					for j in givens:
						totalPos[0] += self.points[j].x
						totalPos[1] += self.points[j].y
					#next j
					avg = (totalPos[0] / len(givens), totalPos[1] / len(givens))
					self.points[i].x, self.points[i].y = avg
				#end if
				if not (False in knownPos): break
			#next i
			if not (False in knownPos):
				#print("Took %i tries" % (roundTry + 1))
				break
			#end if
		#next roundTry
	#end def sortBranch(r0=40)
	
	def center(self):
		"""
		Takes the average and puts the center there. This doesn't work with 
		clusters of nodes with nodes far off. This works when you want to bias 
		toward a cluster.
		"""
		if len(self.points) == 0:
			return
		#end if
		mean = Vector2(0,0)
		for point in self.points:
			mean += point
		#next point
		mean /= (1.0 * len(self.points))
		goto = Vector2(self.width/2.0, self.height/2.0)
		diff = goto - mean
		for point in self.points:
			point.x += diff.x
			point.y += diff.y
		#next point
	#end def center()
	
	def centerBound(self):
		"""
		Instead of averaging, this puts the center based on the mean of the 
		bounds. This works with pretty much all graphs.
		"""
		if len(self.points) == 0:
			return
		#end if
		bounds = self.bounds()
		mean = (Vector2(*bounds[:2]) + Vector2(*bounds[2:])) / 2.0
		goto = Vector2(self.width/2.0, self.height/2.0)
		diff = goto - mean
		for point in self.points:
			point.x += diff.x
			point.y += diff.y
		#next point
	#end def centerBound()
	
	def move(self, x, y):
		"""
		Move everything on the map to offset x, y.
		"""
		if len(self.points) == 0:
			return
		#end if
		for point in self.points:
			point.x += x
			point.y += y
		#next point
	#end def move(x, y)

	def bounds(self):
		"""
		Find the maximum extents.
		returns (x_min, y_min, x_max, y_max)
		"""
		if len(self.points) == 0:
			return
		#end if
		# x_min, y_min, x_max, y_max
		minp = self.points[0] + Vector2(0, 0)
		maxp = minp + Vector2(0, 0)
		for point in self.points:
			if point.x < minp.x: minp.x = point.x
			if point.y < minp.y: minp.y = point.y
			if point.x > maxp.x: maxp.x = point.x
			if point.y > maxp.y: maxp.y = point.y
		#next point
		return (minp.x, minp.y, maxp.x, maxp.y)
	#end def bounds()
	
	def squash(self):
		mean = Vector2(0,0)
		for point in self.points:
			#mean[0] += point.x
			#mean[1] += point.y
			mean += point
		#next point
		#mean[0] /= (1.0 * len(self.points))
		#mean[1] /= (1.0 * len(self.points))
		mean /= (1.0 * len(self.points))
		#goto = [self.width/2.0, self.height/2.0]
		goto = Vector2(self.width/2.0, self.height/2.0)
		#diff = [goto[0]-mean[0], goto[1]-mean[1]]
		diff = goto - mean
		for point in self.points:
			#point.x += diff[0]
			#point.y += diff[1]
			for conn in point.conns:
				point.x += diff.x
				point.y += diff.y
			#next conn
		#next point
	#end def squash()
	
	def split(self):
		"""
		Return a list of independent graphs.
		"""
		output = [set() for p in self.points]
		revMap = {}
		for i in range(len(self.points)):
			revMap[self.points[i]] = i
		#next i
		# First step: merge all connections together.
		for i in range(len(self.points)):
			ourSlot = None
			for conn in self.points[i].conns:
				j = revMap[conn]
				if len(output[j]) > 0:
					if ourSlot != None:
						# Incorporate their useless map.
						output[ourSlot].update(output[j])
						output[j] = set()
					else:
						ourSlot = j
					#end if
					#print(self.points[i].name, 'going to', self.points[j].name)
				#end if
			#next conn
			if ourSlot == None:
				output[i].update([self.points[i]] + self.points[i].conns)
			else:
				output[ourSlot].update([self.points[i]] + self.points[i].conns)
			#end if
		#next i
		# this leaves us with at most a handful of groups that need to be merged.
		# Look for things to merge.
		for i in range(len(self.points)):
			if len(output[i]) > 0:
				toMerge = []
				for p in output[i]:
					#j = revMap[p]
					for k in range(len(self.points)):
						# Don't try to merge with yourself.
						if i == k: continue
						if k in toMerge: continue
						if p in output[k]:
							# Incorporate their useless map.
							toMerge.append(k)
						#end if
					#next k
				#next p
				for k in toMerge:
					output[i].update(output[k])
					output[k] = set()
				#next k
			#end if
		#next i
		while set() in output:
			output.remove(set())
		#loop
		return output
	#end def split()
	
	def __str__(self):
		return self.drawSvg()
	#def __str__(self):
	
	def drawSvg(self):
		output = """<?xml version="1.0" standalone="no"?>
<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">"""
		output += '\n<svg width="' + str(self.width) + '" height="' + str(self.height) + '" viewBox="0 0 ' + str(self.width) + ' ' + str(self.height) + '" xmlns="http://www.w3.org/2000/svg" version="1.1" xmlns:xlink="http://www.w3.org/1999/xlink">\n'
		output += '<title>' + self.title + '</title>\n'
		output += '<desc>' + self.desc + '</desc>\n'
		output += '<rect x="1" y="1" width="' + str(self.width-2) + '" height="' + str(self.height-2) + '" fill="none" stroke="blue" />\n'
		
		for point in self.points:
			tpos = point + Vector2(8, 8)
			# Draw each point
			#output += "<!--<text x='" + ('%3.3f' % tpos.x) + "' y='" + ('%3.3f' % tpos.y) + "'>" + point.name + "</text>-->\n"
			output += "<circle cx='" + ('%3.3f' % point.x) + "' cy='" + ('%3.3f' % point.y) + "' r='" + ('%3.1f' % point.size) + "' fill='" + point.color + "' stroke='grey' stroke-width='1' />\n"
			# Draw each conn
			for conn in point.conns:
				#output += '<!-- ' + str(point) + ' to ' + str(conn) + '-->\n'
				output += "<path d='M " + ('%3.3f' % point.x) + " " + ('%3.3f' % point.y) + " L " + ('%3.3f' % conn.x) + " " + ('%3.3f' % conn.y) + "' fill='none' stroke='" + str(self.line_color) + "' stroke-width='" + str(self.line_width) + "' />\n"
			#next conn
		#next point
		# A layer of text above the connections.
		output += '<g>'
		for point in self.points:
			tpos = point + Vector2(8, 8)
			if point.url != '':
				output += "<a xlink:href='" + point.url.replace('&', '&amp;') + "'>"
			#end if
			output += "<text x='" + ('%3.3f' % tpos.x) + "' y='" + ('%3.3f' % tpos.y) + "' font-size='11'>" + point.name.replace('&', '&amp;') + "</text>\n"
			if point.url != '':
				output += "</a>\n"
			#end if
		#next point
		output += '</g>'
		output += '</svg>'
		return output
	#end def drawSvg()

	def saveJson(self):
		"""
		save JSON output
		Difficult because of the design, but it works.
		"""
		# TODO: Decide if this needs to be a wider dependency.
		import json
		output = '{"nodes":['
		conns = ''
		alreadyConn = set()
		for point in self.points:
			output += '[%s, %i, %i], ' % (json.dumps(point.name), point.x, point.y)
			for conn in point.conns:
				# Don't add a duplicate connection.
				# NOTE: This assumes dual connection, but that MUST be true.
				# TODO: A better way to do this is i < j
				if conn in alreadyConn: continue
				conns += '[%s, %s], ' % (json.dumps(point.name), json.dumps(conn.name))
			#next conn
			alreadyConn.add(point)
		#next point
		# Strip comma.
		output = output[:-2]
		output += '], "conns":[' + conns[:-2]
		# Add end bracket.
		output += ']}'
		return output
	#end def saveJson(self)

	def loadJson(self, jsonData):
		"""
		load JSON string
		Difficult because of the design, but it works.
		No error handling, this should be silly.
		"""
		# TODO: Decide if this needs to be a wider dependency.
		import json
		x = json.loads(jsonData)
		nameObjMap = {}
		for n in x['nodes']:
			# n is in the format [name, x, y]
			nameObjMap[n[0]] = len(self.points)
			self.points.append(Point(n[0], None, n[1], n[2]))
		#next n
		for c in x['conns']:
			# c in in the format [name, name]
			objIndex1 = nameObjMap[c[0]]
			objIndex2 = nameObjMap[c[1]]
			# Dupes easily.
			if self.points[objIndex2] in self.points[objIndex1].conns: continue
			self.points[objIndex1].addConn(self.points[objIndex2])
		#next c
	#end def loadJson(self, jsonData)
	
#end class Map

class Vector2(object):
	def __init__(self, x=0, y=0):
		self.x = x
		self.y = y
	
	def __str__(self):
		return ('%3.3f' % self.x) + ', ' + ('%3.3f' % self.y)
	#end def __str__()
	
	def __add__(self, vec):
		return Vector2(self.x + vec.x, self.y + vec.y)
	
	def __sub__(self, vec):
		return Vector2(self.x - vec.x, self.y - vec.y)
	
	def __mul__(self, other):
		return Vector2(self.x * other, self.y * other)
	
	def __truediv__(self, other):
		#if flt == 0:
		#	return Vector2(0, 0)
		#end if
		return Vector2(self.x / other, self.y / other)
	
	__div__ = __truediv__
	#def __iadd__(self, vec):
	#	v = Vector2(self.x + vec.x, self.y + vec.y)
	#	return v
	
	#def __idiv__(self, flt):
	#	v = Vector2(self.x / flt, self.y / flt)
	#	return v
	
	def length(self):
		return sqrt((self.x * self.x) + (self.y * self.y))
	#end def length()
	
	def dot(self, other):
		return (self.x * other.x) + (self.y * other.y)
	#end def dot(other)
	
#end class Vector2

class Point(Vector2):
	def __init__(self, name, conns=None, x=380, y=180, color='#6666ff', url='', size=4):
		Vector2.__init__(self, x, y)
		self.name = name
		self.url = url
		self.conns = conns
		self.color = color
		self.size = size
		if conns == None: self.conns = []

	def __str__(self):
		return str(self.name) + ' ' + Vector2.__str__(self)
	#end def __str__()
	
	def addConn(self, conn, reConn=True):
		#print 'adding conn?', self.name, conn
		self.conns.append(conn)
		if(reConn): conn.addConn(self, False)

	def setColor(self, value):
		""" Set color to a hex value like #ff0000 """
		self.color = value
#end class Point(Vector2)

def createMap(seed_id=3243):
	seed(seed_id)
	a = Map()
	b = []
	for i in range(8):
		b.append(Point(chr(ord('b')+i)))
	#next i
	for i in range(7):
		b[i].addConn(b[i+1])
	#next i
	addConn1 = randint(0,len(b)-1)
	addConn2 = (addConn1 + randint(0,len(b))) % len(b)
	if addConn1 == addConn2:
		# Possible
		addConn2 = (addConn1 ^ 7) % len(b)
	if addConn1 == addConn2:
		# Unlikely
		addConn1 = 0
	if addConn1 == addConn2:
		# Impossible
		addConn2 = 1
	#print b, addConn1, addConn2
	b[addConn1].addConn(b[addConn2])
	a.addPoints(b)
	for i in range(4):
		a.randomize()
		a.sort1()
		#a.sort1()
		#a.sort1()
		#a.sort1()
		#a.sort1()
	#next i
	return a
#end def createMap([seed_id])

def main():
	print(createMap())
#end def main()

if __name__ == '__main__':
	main()
#end if
