#!/usr/bin/env python

"""
mtvmap1.py
by Javantea
Oct 27, 2008
Based on pwmap2.py

Maps artists/videos using MTV API

Each artist has a list of related artists and a list of videos. 
This is a perfect system to map.

This model is wrong:
white_stripes -- white_stripes-related -- ranconteurs
 \-- white_stripes-videos
"""

import urllib.request, urllib.error, urllib.parse
import re

from sys import path
path.append('.')
import mapmake1
from random import seed
from os import path
from time import sleep
from xml.etree import ElementTree

# We use urlopen and file depending on protocol.
# http://api.programmableweb.com/

class ArtistMap(mapmake1.Map):
	def __init__(self, url='/1/artist/browse/w', seed_id = 234, level=1, width=800, height=600, title='Small Wide World MTV Mashup'):
		seed(seed_id)
		# Width, height, line_width = 1.
		mapmake1.Map.__init__(self, None, width, height, 1, '#9999ff')
		self.title = title
		self.p = []
		self.i = 0
		self.names = {}
		self.apikey = ''
		
		# Grab a given number of pages from API.
		#for pageno in range(pages):
		self.parseUrl(url, level)
		#next pageno
		
		# Change the size of each point depending on how many links it has.
		for point in self.points:
			size = len(point.conns) * 2
			if size > 8:
				# in: out, 8: 8, 9: 8.125, 20: 9.5
				size = 7 + size/8.0
			#end if
			point.size = size
		#next point
		
		# Organize pushes the points into position depending on their connections.
		self.organize()
	#def __init__(seed_id = 234, levels=1)

	def parseUrl(self, url='/1/artist/browse/w', level=1):
		p_new = self.i
		pageqs = ''
		#if pageno != 1: pageqs = '&page=' + str(pageno)
		filename = 'data/' + url.replace('/', '_')[1:] + '.xml'
		#print '<!-- Grabbing', 'http://api.programmableweb.com' + url, '-->'
		if not path.exists(filename):
			# Need an absolute url.
			try:
				data = urllib.request.urlopen('http://api.mtvnservices.com' + url + pageqs).read()
				open(filename, 'wb').write(data)
				sleep(1.0)
			except urllib.error.URLError as e:
				print('Warning: Could not grab artist data', url, ':', e)
				#skip over adding this friend's friends.
				return
			#end try
		#end if
		
		# Use XML
		# Thanks to Yahoo Developer for the idea:
		# http://developer.yahoo.com/python/python-xml.html
		tree1 = ElementTree.parse(filename)
		root = tree1.getroot()
		#print root[0].tag, root[0].text
		NS   = "http://www.w3.org/2005/Atom"
		MRSSNS = "http://search.yahoo.com/mrss/"
		this_artist = [i.text for i in root.findall('{%s}title' % (NS, ))]
		urls = [((i.get('rel') == 'self') and i.get('href')) for i in root.findall('{%s}entry/{%s}link' % (NS, NS))]
		artists = [i.text for i in root.findall('{%s}entry/{%s}title' % (NS, NS))]
		self_urls = []
		for lurl in urls:
			if lurl:
				self_urls.append(lurl)
			#end if
		#next url
		name_pts = []
		
		#print 'u', url
		#print 't', this_artist
		
		artist_name = ''
		if ((len(this_artist) == 1) and this_artist[0] != None):
			if this_artist[0][-16:] == ' Related Artists':
				artist_name = this_artist[0][:-16]
			else:
				artist_name = this_artist[0]
			#end if
		else:
			print("Missing artist_name:", this_artist)
		#end if
		
		if artist_name not in list(self.names.keys()):
			self.p.append(mapmake1.Point(artist_name, color='#99ccff', url='http://api.mtvnservices.com' + url))
			self.names[artist_name] = self.i
			name_pts.append(self.i)
			artist_i = self.i
			self.i += 1
		else:
			artist_i = self.names[artist_name]
		#end if
		
		# They must be crazy putting null at the first of all.
		if artists[0] == None:
			artists.pop(0)
			self_urls.pop(0)
		#end if
		
		j = 0
		for artist in artists:
			# What are they thinking putting a <title /> in here?
			if artist == None:
				j += 1
				continue
			#end if
			if artist not in list(self.names.keys()):
				#print 'm', mashup
				self.p.append(mapmake1.Point(artist, color='#99ccff', url=self_urls[j]))
				self.names[artist] = self.i
				name_pts.append(self.i)
				self.p[artist_i].addConn(self.p[self.i])
				self.i += 1
			else:
				name_pts.append(self.names[artist])
			#end if
			
			# There are two links per mashup, we want the first
			j += 1
		#next artist
		
		if level > 0:
			# Recursive Call... Each level gets smaller by one.
			#apis2d = [[j.text for j in i.findall('{%s}content/{%s}mashup/{%s}apis/{%s}api/{%s}name' % (NS, PWNS, NS, NS, NS))] for i in root.findall('{%s}entry' % (NS,))]
			#api_urls2d = [[j.text for j in i.findall('{%s}content/{%s}mashup/{%s}apis/{%s}api/{%s}url' % (NS, PWNS, NS, NS, NS))] for i in root.findall('{%s}entry' % (NS,))]
			for url in self_urls[:4]:
				self.parseUrl(url.replace('http://api.mtvnservices.com', '') + 'related', level - 1)
			#next url
		#end if
		
		# The new ones in self.p are from p_new to the end
		# verified, fixed a bug.
		#print p_new, len(self.p)
		self.addPoints(self.p[p_new:])
	#end def parseUrl([url], [seed_id], levels)
	
	def sort2(self, dist1 = 100, f = 0.5):
		# We can override sort2 to do dist1 = 170 instead of 70.
		mapmake1.Map.sort2(self, dist1, f)
	#end def sort2(self, dist1 = 70, f = 0.5)
	
	def organize(self):
		self.randomize(int((self.width + self.height) / 1.4))
		for i in range(40):
			self.sort2((self.width + self.height) / 15)
			if i % 30 == 0:
				self.sort3()
			#end if
			if i % 32 == 0:
				self.sort4()
			#end if
			if i % 25 == 0:
				self.randomize(200)
			#end if
			#a.sort1()
			#a.sort1()
			#a.sort1()
			#a.sort1()
		#next i
		for i in range(25):
			self.sort4()
		#next i
		#m.sort3()
		self.center()
	#def organize()
	
	#def __str__(self):
	#	return str(self.m)
	#end def __str__()
#end class ApiMashupMap(mapmake1.Map)

def main(seed_id = 4723):
	url = '/1/artist/white_stripes/related'
	from sys import argv
	if len(argv) > 1:
		url = argv[1]
	#end if
	m = ArtistMap(url, seed_id, 1, 800, 800, 'MTV White Stripes Map')
	#for point in m.points:
	#	print 'p', point.name
	#next point
	print(str(m).encode('utf-8'))
#end def main()

if __name__ == '__main__':
	main()
#end if
