<?php
/*
 * Small Wide World
 * by Joel R. Voss aka. Javantea
 * Copyright 2007 AltSci Concepts
 * June 28, 2007
 *
 * Port to C from Python.
 */

/*
Map maker
by Javantea
April 26, 2007
Ported to PHP from C++: June 29, 2007

Description:
An idea to make maps.
1: Place all points at the center.
2: Move each point randomly.
3: Move each point closer to it's connected points.
4: Goto 2
5: Profit!

*/
define(RAND_MAX, 300000);

function randint($min, $max)
{
	$r = rand($min, $max);
	//cout << "r = " << r << ", RM = " << RAND_MAX << ", max=" << max << ", min=" << min << endl;
	//print "$r, ";
	//$f = $r / RAND_MAX;
	//cout << "f = " << f << endl;
	return (int)$r; //(int)($min + $f * ($max - $min));
}

/*
Base class for all maps (friend, object, route).
Write the hard math and physics here once and use it by calling once.
*/
// points=None, width=640, height=480, line_width=2
class Map
{

public static $name = '';
public static $points = NULL;
public static self::$width = 640;
public static self::$height = 480;
public static self::$line_width = 2;
public static self::$selectedPoint = NULL;

function Map($init_points, $init_name, $init_width, $init_height, $init_line_width)
{
	if($init_points != NULL)
	{
		self::$points = $init_points;
	}
	else
	{
		self::$points = array();
	}
	//# There's a bug in python where if I do points=[] above, it 
	//# thinks it's a static when it's really really not.
	//if(points == None) self.points = [];
	self::$name = $init_name;
	self::$width  = $init_width;
	self::$height = $init_height;
	self::$line_width = $init_line_width;
	self::$selectedPoint = NULL;
}

function addPoint($point)
{
	self::$points[] = ($point);
}

function addPoints($add_points)
{
	for($i = 0; $i < count($add_points); $i++)
	{
		self::$points[] = $add_points[$i];
	}
}

// amount = 180
function randomize($amount=180)
{
	for($i = 0; $i < count(self::$points); $i++)
	{
		self::$points[$i]->x += randint(-$amount, $amount);
		self::$points[$i]->y += randint(-$amount, $amount);
	}
	//#next point
}
//#end void randomize()

function drag($x, $y)
{
	if(self::$selectedPoint == NULL)
	{
		return false;
	}
	//#end if
	$r = new Vector2(x, y);
	$d_len = ($r->sub(self::$selectedPoint)).length();
	if($d_len > 10)
	{
		self::$selectedPoint->x = x;
		self::$selectedPoint->y = y;
		return true;
	}
	//#end if
	return false;
}
//#end void drag(x, y)

function click($x, $y)
{
	$cp = new Vector2(x, y);
	$min_dr = -1.0;
	$min_dr_p = NULL;
	for($i = 0; $i < count(self::$points); $i++)
	{
		$dr = $cp->sub(self::$points[$i]);
		$dr_len = $dr.length();
		if(($min_dr < 0) || ($dr_len < $min_dr))
		{
			$min_dr = $dr_len;
			$min_dr_p = self::$points[$i];
		}
		//#end if
	}
	//#next point
	if($min_dr_p != NULL)
	{
		print "Setting color!\n";
		$min_dr_p->setColor("#cc0000");
		if(self::$selectedPoint != NULL)
		{
			self::$selectedPoint->setColor("#0000ff");
		}
		//#end if
		self::$selectedPoint = $min_dr_p;
	}
	//#end if
}
//#end void click(x, y)

function sort1()
{
	for($i = 0; $i < count(self::$points); $i++)
	{
		for($j = 0; $j < count(self::$points[$i]->conns); $j++)
		{
			$conn = self::$points[$i]->conns[$j];
			$dr = $conn->sub(self::$points[$i]);
			//#(dx, dy) = (conn->x - point->x, conn->y - point->y)
			//points[$i] += dr * 0.4;
			self::$points[$i]->x += $dr->x * 0.4;
			self::$points[$i]->y += $dr->y * 0.4;
		}
		//#next conn
	}
	//#next point
}
//#end void sort1()

// dist1 = 70, f = 0.5
function sort2($dist1=70, $f=0.5)
{
	/*
	Use springs to decide the position of each point.
	*/
	for($i = 0; $i < count(self::$points); $i++)
	{
		for($j = 0; $j < count(self::$points[$i]->conns); $j++)
		{
			$conn = self::$points[$i]->conns[$j];
			//#(dx, dy) = (conn->x - point->x, conn->y - point->y)
			$dr = $conn->sub(self::$points[$i]);
			//#dr_len = sqrt(dx*dx + dy*dy)
			$dr_len = $dr.length();
			//#print '	dr:', dr
			if($dr_len == 0.0)
			{
				self::$points[$i]->x += 5;
				self::$points[$i]->y += 5;
				$dr_len = sqrt(5*5 + 5*5);
				//#print 'zero dr is now:', dr
			}
			//#end if
			//#print 'non-zero dr:', dr
			$m = 1;
			$a_len = ($f/$m) * ($dr_len - $dist1);
			//#dr_hat = (dx/dr, dy/dr)
			$dr_hat = $dr->div($dr_len);
			//#a = (a_len*dr_hat[0], a_len*dr_hat[1]/m)
			$a = $dr_hat->mul($a_len / $m);
			$dt = 0.6;
			//#dp = (0.5*dt*dt*a[0], 0.5*dt*dt*a[1])
			$dp = $a * (0.5*$dt*$dt);
			self::$points[$i]->x += $dp->x;
			self::$points[$i]->y += $dp->y;
		}
		//#next conn
	}
	//#next point
}
//#end void sort2()

function sort3()
{
	/*
	Use springs to decide the position of each point.
	*/
	for($i = 0; $i < count(self::$points); $i++)
	{
		if(self::$points[$i]->conns.size() <= 1) continue;
		//#avgx = 0
		//#avgy = 0
		$avg = new Vector2(0, 0);
		for($j = 0; $j < count(self::$points[$i]->conns); $j++)
		{
			//#avgx += conn->x
			//#avgy += conn->y
			$avg->iadd((self::$points[$i]->conns[$j]));
		}
		//#next conn
		//#point->x = avgx / len(point.conns)
		//#point->y = avgy / len(point.conns)
		$p = $avg / count(self::$points[$i]->conns);
		self::$points[$i]->x = $p->x;
		self::$points[$i]->y = $p->y;
	}
	//#next point
}
//#end void sort3()

function sort4()
{
	/*
	Use a style of reverse attack on non-conns that are close.
	*/
	for($i = 0; $i < count(self::$points); $i++)
	{
		for($j = 0; $j < self::$points.size(); $j++)
		{
			// nonconn
			//if(points[$j] in point.conns)
			if(in_array($point[$i]->conn, self::$points[$j]))
			{
				continue;
			}
			//#end if
			$dr = self::$points[$i]->sub(self::$points[$j]);
			$dr_len = $dr.length();
			//#print 'dr_len =', dr_len
			if($dr_len < 50)
			{
				if($dr_len == 0)
				{
					$dr = new Vector2(1, 1);
					$dr_len = sqrt(2);
				}
				//#end if
				$dr *= 4.0 / $dr_len;
				self::$points[$i]->x += $dr->x;
				self::$points[$i]->y += $dr->y;
			}
			//#end if
		}
		//#next nonconn
	}
	//#next point
}
//#end void sort4()

function center()
{
	$mean = new Vector2(0,0);
	for($i = 0; $i < count(self::$points); $i++)
	{
		//#mean[0] += point->x
		//#mean[1] += point->y
		$mean->iadd(self::$points[$i]);
	}
	//#next point
	//#mean[0] /= (1.0 * len(self.points))
	//#mean[1] /= (1.0 * len(self.points))
	$mean /= (1.0 * count(self::$points));
	//#goto = [self.width/2.0, self.height/2.0]
	$goto_pos = new Vector2((int)(self::$width/2.0), (int)(self::$height/2.0));
	//#diff = [goto[0]-mean[0], goto[1]-mean[1]]
	$diff = $goto_pos->sub($mean);
	//#print 'center diff', diff
	for($i = 0; $i < count(self::$points); $i++)
	{
		//#point->x += diff[0]
		//#point->y += diff[1]
		self::$points[$i]->x += $diff->x;
		self::$points[$i]->y += $diff->y;
	}
	//#next point
}
//#end void center()

function squash()
{
	$mean = new Vector2(0,0);
	for($i = 0; $i < count(self::$points); $i++)
	{
		//#mean[0] += point->x
		//#mean[1] += point->y
		$mean->iadd(self::$points[$i]);
	}
	//#next point
	//#mean[0] /= (1.0 * len(self.points))
	//#mean[1] /= (1.0 * len(self.points))
	$mean /= (1.0 * count(self::$points));
	//#goto = [self.width/2.0, self.height/2.0]
	$goto_pos = new Vector2((int)(self::$width/2.0), (int)(self::$height/2.0));
	//#diff = [goto[0]-mean[0], goto[1]-mean[1]]
	$diff = $goto_pos->sub($mean);
	for($i = 0; $i < count(self::$points); $i++)
	{
		//#point->x += diff[0]
		//#point->y += diff[1]
		// Note that this is a bug in center, but I'm leaving it in as squash.
		for($j = 0; $j < count(self::$points[$i]->conns); $j++)
		{
			self::$points[$i]->x += $diff->x;
			self::$points[$i]->y += $diff->y;
		}
		//#next conn
	}
	//#next point
}
//#end void squash()


function str()
{
	return $this->drawSvg();
}
//#void __str__(self):

function drawSvg()
{
	$output = "<?xml version=\"1.0\" standalone=\"no\"?>\n" .
		"<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">";
	$output .= "\n<svg width=\"" . self::$width . "\" height=\"" . self::$height . "\" viewBox=\"0 0 " . self::$width . " " . self::$height . "\" xmlns=\"http://www.w3.org/2000/svg\" version=\"1.1\">\n";
	$output .= "<title>Network Map 4 svg</title>\n";
	$output .= "<desc>Test of drawing a network map with svg</desc>\n";
	$output .= "<rect x=\"1\" y=\"1\" width=\"" . (self::$width-2) . "\" height=\"" . (self::$height-2) . "\" fill=\"none\" stroke=\"blue\" />\n";
	
	for($i = 0; $i < count(self::$points); $i++)
	{
		$tpos = self::$points[$i]->add(new Vector2(8, 8));
		//# Draw each point
		$output .= "<!--<text x='" . ($tpos->x) . "' y='" . ($tpos->y) . "'>" . self::$points[$i]->name . "</text>-->\n";
		$output .= "<circle cx='" . (self::$points[$i]->x) . "' cy='" . (self::$points[$i]->y) . "' r='4' fill='" . self::$points[$i]->color . "' stroke='black' stroke-width='1' />\n";
		//# Draw each conn
		for($j = 0; $j < count(self::$points[$i]->conns); $j++)
		{
			$output .= "<!-- " . self::$points[$i]->str() . " to \n";// + points[$i]->conns[$j]->str() + "-->\n";
			$output .= "<path d='M " . (self::$points[$i]->x) . " " . (self::$points[$i]->y) . " L " . (self::$points[$i]->conns[$j]->x) . " " . (self::$points[$i]->conns[$j]->y) . "' fill='none' stroke='blue' stroke-width='" . (self::$line_width) . "' />\n";
		}
		//#next conn
	}
	//#next point
	//# A layer of text above the connections.
	$output .= "<g>\n";
	for($i = 0; $i < count(self::$points); $i++)
	{
		$tpos = self::$points[$i]->add(new Vector2(8, 8));
		$output .= "<text x='" . $tpos->x . "' y='" . $tpos->y . "'>" + $point->name + "</text>\n";
	}
	//#next point
	$output .= "</g>\n";
	$output .= "</svg>\n";
	return $output;
}
// #end void drawSvg()

}
// #end class Map

class Vector2
{
	var $x;
	var $y;
//class Vector2:
// x=380, y=180
function Vector2($init_x=380, $init_y=180)
{
	$x = $init_x;
	$y = $init_y;
}

/*function Vector2(Point *p)
{
	$x = $p->x;
	$y = $p->y;
}*/

function str()
{
	$s = ($x) . ", " . ($y);
	return $s;
}
//#end void __str__()

function add($v)
{
	return new Vector2($x + $v->x, $y + $v->y);
}

function sub($v)
{
	return new Vector2($x - $v->x, $y - $v->y);
}

function mul($f)
{
	return new Vector2(($x * $f), ($y * $f));
}

function div($f)
{
	return new Vector2(($x / $f), ($y / $f));
}

function iadd($v)
{
	$x += $v->x;
	$y += $v->y;
	return $this;
}

function isub($v)
{
	$x -= $v->x;
	$y -= $v->y;
	return $this;
}

function imul($f)
{
	$x *= $f;
	$y *= $f;
	return $this;
}

function idiv($f)
{
	$x /= $f;
	$y /= $f;
	return $this;
}

//#void __iadd__(vec):
//#	v = Vector2(self->x + vec->x, self->y + vec->y)
//#	return v

//#void __idiv__(flt):
//#	v = Vector2(self->x / flt, self->y / flt)
//#	return v

function length()
{
	return sqrt($x * $x + $y * $y);
}

}
//#end class Vector2

class Point extends Vector2
{
var self::$name;
var $x;
var $y;
var $conns;
var $color;

//class Point(Vector2):
//name, conns=None, x=380, y=180, color='//#0000ff'
function Point($init_name, $init_x=380, $init_y=180, $init_conns=NULL, $init_color='#0000ff')
{
	Vector2::Vector2($init_x, $init_y);
	self::$name = $init_name;
	if($init_conns != NULL)
	{
		$conns = $init_conns;
	}
	else
	{
		$conns = array();
	}
	$color = $init_color;
	//if conns == None: self.conns = []
}

function str()
{
	//cout << name << endl;
	$s = self::$name . " " . ($x) . ", " . ($y); //Vector2::str();
	return $s;
}
//#end void __str__()

// conn, reConn=True
function addConn($conn, $reConn=true)
{
	//#print 'adding conn?', self.name, conn
	//cout << "adding conn " << name << ", " << conn->name << ", " << reConn << ", " << conns.size() << endl;
	$conns[] = ($conn);
	if($reConn)
	{
		$conn->addConn($this, false);
	}
}

function setColor($value)
{
	// Set color to a hex value like //#ff0000 //
	$color = $value;
}

}
//#end class Point(Vector2)

function createMap($seed_id=3211)
{
	srand($seed_id);
	$a = new Map(NULL, "Test Map", 640, 480, 2);
	$b = array();
	for($i = 0; $i < 8; $i++)
	{
		self::$name = "b";
		self::$name[0] = chr(ord("b") + i);
		$b[] = new Point(self::$name, 320, 240);
	}
	//#next i
	for($i = 0; $i < 7; $i++)
	{
		$b[$i]->addConn(($b[$i+1]));
	}
	//#next i
	$addConn1 = randint(0,count($b)-1);
	$addConn2 = (addConn1 + randint(0,count($b))) % count($b);
	//cout << addConn1 << ", " << addConn2 << endl;
	if($addConn1 == $addConn2)
	{
		//# Possible
		$addConn2 = ($addConn1 ^ 7) % count($b);
	}
	if($addConn1 == $addConn2)
	{
		//# Unlikely
		$addConn1 = 0;
	}
	if($addConn1 == $addConn2)
	{
		//# Impossible
		$addConn2 = count($b) - 1;
	}
	//#print b, addConn1, addConn2
	$b[$addConn1]->addConn(($b[$addConn2]));
	$a->addPoints($b);
	for($i = 0; $i < 4000; $i++)
	{
		$a->randomize();
		$a->sort1();
		//#a.sort1()
		//#a.sort1()
		//#a.sort1()
		//#a.sort1()
	}
	//#next i
	return $a;
}
//#end void createMap([seed_id])

function main()
{
	$m = createMap();
	print $m->str();
}
//#end void main()

//if __name__ == '__main__':
//	main()
//#end if

main();

?>
