#!/usr/bin/env python3
"""
rsvg_map2.py
by Javantea

Draw maps with rsvg.
This cool library can easily draw any svg from file or memory.
This is the proper way to make dynamic, editable svg systems in Python.
Eventually this frame will be used to create many cool projects such as AltSci 
Composer.
"""
from __future__ import print_function
import sys
import cairo
import rsvg
import gtk
from time import sleep
from threading import Thread

class DisplaySVG(gtk.Window):
	
	BORDER_WIDTH = 1
	#pixmap = None
	
	clx, cly = -2, -2
	
	def __init__(self, name='rsvg Map Renderer in Python', object_init=None, object_args=None):
		gtk.Window.__init__(self)
		self.connect("delete-event", self.delete_cb)
		self.set_title(name)
		#self.set_icon_from_file('bldg1_icon32.png')
		
		self.svg_file = 'mapmake1_out.svg'
		self.data = ''
		self.svg = None
		self.seed = '222'
		self.object_init = object_init
		args1 = list(object_args)
		# The second one is always seed.
		if len(object_args) >= 2:
			self.seed = args1[1]
		#end if
		self.object_args = tuple(args1)
		if object_init == None:
			if len(object_args) >= 1:
				filename = object_args[0]
			else:
				print("Looking for a file or an object.")
				return
			#end if
			self.svg = rsvg.Handle(file=filename)
		else:
			# rsvg can be written to from memory.
			self.map1 = object_init(*object_args)
			self.data = str(self.map1)
			self.svg = rsvg.Handle()
			self.svg.write(self.data)
			self.svg.close()
			#filename = 'temp1.svg'
			#file(filename, 'w').write(s)
			#raise SystemExit("need svg file")
		#end if
		
		#win.connect("expose-event", expose_cairo, svg)
		
		hbox1 = gtk.HBox()
		vbox1 = gtk.VBox()
		drawingarea = gtk.DrawingArea()
		self.svgview1 = gtk.TextView()
		buffer_1 = self.svgview1.get_buffer()
		#print buffer_1
		sw = gtk.ScrolledWindow()
		sw.set_shadow_type(gtk.SHADOW_ETCHED_IN)
		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		sw.add(self.svgview1)
		self.svgview1.set_size_request(300, 600)
		self.attr_table = gtk.Table(8, 3)
		self.row = {}
		self.addRow('Seed', self.seed, '', single=True)
		"""self.attr_table.attach(gtk.Label('Width'), 1, 2, 0, 1)
		self.attr_table.attach(gtk.Label('Height'), 2, 3, 0, 1)
		self.row = {}
		self.addRow('Pixel', bldg.pixels_w, bldg.pixels_h)
		self.addRow('Floor', bldg.a, bldg.h)
		self.addRow('Window', bldg.ww, bldg.wh)
		self.addRow('Door', bldg.dw, bldg.dh)
		self.addRow('Padding', bldg.pw, bldg.ph)
		self.addRow('W_m / n_i', bldg.m, bldg.ni)
		self.addRow('Floors', '', bldg.f)"""
		
		viewb1 = gtk.Button("Test SVG")
		viewb1.connect('button_press_event', self.redraw)
		genb1 = gtk.Button("Gen")
		genb1.connect('button_press_event', self.regen)
		savesb1 = gtk.Button("Save SVG")
		savesb1.connect('button_press_event', self.saveSvg)
		animateb1 = gtk.Button("Animate")
		animateb1.connect('button_press_event', self.animate)
		centerb1 = gtk.Button("Center")
		centerb1.connect('button_press_event', self.center)
		sortb2 = gtk.Button("Sort 2")
		sortb2.connect('button_press_event', self.sort2)
		sortb3 = gtk.Button("Sort 3")
		sortb3.connect('button_press_event', self.sort3)
		sortb4 = gtk.Button("Sort 4")
		sortb4.connect('button_press_event', self.sort4)
		sortb5 = gtk.Button("Sort 5")
		sortb5.connect('button_press_event', self.sort5)
		but_box2 = gtk.HButtonBox()
		but_box3 = gtk.HButtonBox()
		but_box4 = gtk.HButtonBox()
		but_box1 = gtk.VBox()
		but_box2.add(viewb1)
		but_box2.add(genb1)
		but_box2.add(savesb1)
		but_box3.add(animateb1)
		but_box3.add(centerb1)
		but_box3.add(sortb2)
		but_box4.add(sortb3)
		but_box4.add(sortb4)
		but_box4.add(sortb5)
		but_box1.add(but_box2)
		but_box1.add(but_box3)
		but_box1.add(but_box4)
		
		vbox1.add(self.attr_table)
		vbox1.add(sw)
		vbox1.add(but_box1)
		hbox1.add(drawingarea)
		hbox1.add(vbox1)
		self.add(hbox1)
		self.set_default_size(1024, 768)
		
		drawingarea.connect('expose_event', self.expose_cairo)
		drawingarea.connect('button_press_event', self.click_screen)
		drawingarea.connect('motion_notify_event', self.drag_screen)
		drawingarea.set_events(gtk.gdk.BUTTON_PRESS_MASK | gtk.gdk.POINTER_MOTION_MASK)
		drawingarea.set_size_request(self.svg.props.width, self.svg.props.height)
		
		print(self.svg.props.width, self.svg.props.height, self.svg.props.em, self.svg.props.ex)
		self.create_tags(buffer_1)
		self.insert_text(buffer_1, self.data)
		
		self.show_all()
		self.connect("destroy", lambda w: gtk.main_quit())
	#end def __init__()
	
	def getAttr(self):
		"""a = {'pixels_w': 'Pixel_width', 'pixels_h': 'Pixel_height', 
			'a': 'Floor_width', 'h': 'Floor_height', 
			'ww': 'Window_width',  'wh': 'Window_height', 
			'dw': 'Door_width',  'dh': 'Door_height', 
			'pw': 'Padding_width',  'ph': 'Padding_height', 
			'm': 'W_m / n_i_width', 'ni': 'W_m / n_i_height', 
			'f': 'Floors_height'
		}"""
		print(self.row)
		a = {'seed': 'Seed_width'}
		b = {}
		for (k, v) in list(a.items()):
			b[k] = self.row[v].get_text()
		#end if
		return b
	#end def getAttr()
	
	def addRow(self, name, default_width='', default_height='', single=False):
		row = len(self.row) + 1
		self.attr_table.attach(gtk.Label(name), 0, 1, row, row+1)
		pw = gtk.Entry()
		pw.set_width_chars(5)
		self.row[name + '_width'] = pw
		self.attr_table.attach(pw, 1, 2, row, row+1)
		pw.set_text(str(default_width))
		if single == False:
			ph = gtk.Entry()
			ph.set_width_chars(5)
			self.row[name + '_height'] = ph
			self.attr_table.attach(ph, 2, 3, row, row+1)
			ph.set_text(str(default_height))
		#end if
	#end addRow(self, name)
	
	def delete_cb(self, win, event):
		gtk.main_quit()
	
	def expose_cairo(self, win, event):
	
		x, y, w, h = win.allocation
		#style = win.get_style().copy()
		#style.bg[gtk.STATE_NORMAL] = style.white
		#win.set_style(style)
		cr = win.window.cairo_create()
		
		if self.svg != None:
			self.svg.render_cairo(cr)
		
		cr.set_source_color(win.style.fg[win.state])
		cr.rectangle(self.BORDER_WIDTH, self.BORDER_WIDTH,
				w - 2*self.BORDER_WIDTH, h - 2*self.BORDER_WIDTH)
		cr.set_line_width(2.0)
		cr.set_line_join(cairo.LINE_JOIN_ROUND)
		cr.stroke()
		
		# Draw the click.
		cr.rectangle(self.clx-1, self.cly-1, 2, 2)
		cr.stroke()
		
		return True
	
	def drag_screen(self, win, event):
		if event.state & gtk.gdk.BUTTON1_MASK == 0:
			return False
		#end if
		#print 'dragging!'
		x, y, = event.x, event.y
		if self.map1.drag(x, y):
			self.drawData(win, self.map1)
		#end if
		return True
	
	def click_screen(self, win, event):
		print('clicked!')
		if event.button == 1:
			self.clx, self.cly = event.x, event.y
			self.map1.click(event.x, event.y)
			#win.window.invalidate_rect(None, True)
			self.drawData(win, self.map1)
		#end if
		return True
	
	def saveSvg(self, win, event):
		#print 'clicked redraw!'
		# rsvg can be written to from memory.
		#a = bldgs3d.bldg3d()
		#s = str(a)
		d = gtk.FileChooserDialog(title="Save SVG", parent=None, 
			action=gtk.FILE_CHOOSER_ACTION_SAVE, 
			buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
				gtk.STOCK_SAVE, gtk.RESPONSE_OK))
		d.show()
		response = d.run()
		if response == gtk.RESPONSE_CANCEL:
			d.destroy()
			return True
		#end if
		svg_file = d.get_filename()
		d.destroy()
		buffer1 = self.svgview1.get_buffer()
		start, end = buffer1.get_bounds()
		self.svg_data = buffer1.get_text(start, end)
		open(svg_file, 'w').write(self.svg_data)
		print("Saved", svg_file, '.')
		return True
	#end def saveSvg(win)
	
	def redraw(self, win, event):
		"""
		This doesn't mean redraw map1, it means redraw the svg text box.
		"""
		print('clicked redraw!')
		# rsvg can be written to from memory.
		#a = bldgs3d.bldg3d()
		#s = str(a)
		buffer_1 = self.svgview1.get_buffer()
		start, end = buffer_1.get_bounds()
		self.data = buffer_1.get_text(start, end)
		self.svg = rsvg.Handle()
		self.svg.write(self.data)
		self.svg.close()
		win.window.invalidate_rect(None, True)
		return True
	
	def center(self, win, event):
		print('clicked center!')
		#b = self.getAttr()
		#print b
		# rsvg can be written to from memory.
		#self.map1 = mapmake1.createMap(b['seed'])
		self.map1.centerBound()
		self.drawData(win, self.map1)
	
	def sort2(self, win, event):
		print('sort clicked!')
		#b = self.getAttr()
		#print b
		# rsvg can be written to from memory.
		#self.map1 = mapmake1.createMap(b['seed'])
		self.map1.sort2(100)
		self.drawData(win, self.map1)
	
	def sort3(self, win, event):
		print('sort clicked!')
		self.map1.sort3()
		self.drawData(win, self.map1)
	
	def sort4(self, win, event):
		print('sort clicked!')
		self.map1.sort4()
		self.drawData(win, self.map1)
	
	# Newest algorithm
	def sort5(self, win, event):
		print('sort clicked!')
		self.map1.sort5()
		self.drawData(win, self.map1)
	
	def regen(self, win, event):
		print('clicked regen!')
		b = self.getAttr()
		print(b)
		self.seed = b['seed']
		# rsvg can be written to from memory.
		
		args1 = list(self.object_args)
		# The second one is always seed.
		if len(self.object_args) >= 2:
			args1[1] = self.seed
		#end if
		self.object_args = tuple(args1)
		self.map1 = self.object_init(*self.object_args)
		self.drawData(win, self.map1)
		
	def drawData(self, win, map1):
		self.data = str(map1)
		buffer_1 = self.svgview1.get_buffer()
		self.clear_text(buffer_1)
		self.insert_text(buffer_1, self.data)
		#self.data = buffer_1.get_text(start, end)
		self.svg = rsvg.Handle()
		self.svg.write(self.data)
		self.svg.close()
		win.window.invalidate_rect(None, True)
		return True
	#end def regen(win, event)
	
	def animate(self, win, event=None):
		self.win_anim = win
		self.anim_thread = Thread(target=self.animateLoop)
		self.anim_thread.start()
		return True
	#end
	
	def animateLoop(self):
		amount = 20
		for i in range(8):
			self.map1.randomize(amount)
			self.map1.sort2()
			gtk.gdk.threads_enter()
			self.drawData(self.win_anim, self.map1)
			gtk.gdk.threads_leave()
			sleep(0.1)
		#next i
		"""sleep(0.1)
		self.map1.randomize(amount)
		self.map1.sort1()
		self.drawData(win, self.map1)
		sleep(0.1)
		self.map1.randomize(amount)
		self.map1.sort1()
		self.drawData(win, self.map1)
		sleep(0.1)
		self.map1.randomize(amount)
		self.map1.sort1()
		self.drawData(win, self.map1)
		sleep(0.1)"""
		return True
	#end def animate(win, event)
	
	def clear_text(self, text_buffer):
		start, end = text_buffer.get_bounds()
		text_buffer.delete(start, end)
	#end def clear_text(text_buffer)
	
	def create_tags(self, text_buffer):
		text_buffer.create_tag("word_wrap", wrap_mode=gtk.WRAP_WORD)
	#end def create_tags(text_buffer)
	
	def insert_text(self, text_buffer, data):
		iter = text_buffer.get_iter_at_offset(0)
		text_buffer.insert(iter, data)
		start, end = text_buffer.get_bounds()
		text_buffer.apply_tag_by_name("word_wrap", start, end)
	#end def insert_text(text_buffer)
	
#end class DisplaySVG

def main():
	from sys import path
	path.append('.')
	import friendmap2
	seed = '222'
	filename = 'data/dodge_jav1.html'
	if (len (sys.argv) > 1):
		filename = sys.argv[1]
	#end if
	a = DisplaySVG('rsvg Friend Map Renderer in Python', friendmap2.FriendMap, (filename, seed, 1))
	gtk.gdk.threads_init()
	gtk.main()
#end def main()

if __name__ == '__main__':
    main()
#end if
