#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
SWW Brute 3
by Javantea
Nov 8, 2015
Based on SWW Optimize 1
by Javantea
Sept 30, 2015
Based on N Atom molecule using Basin-Hopping Method (aka L-BFGS-B)
by Javantea
July 23, 2014
Based on metropolis_min1.py by Javantea, July 17-22, 2014
Based on s_4atom.py by Javantea, June 24, 2014

"""
from __future__ import print_function
#import numpy as np
#from scipy.optimize import minimize, basinhopping
import mapmake1
import math
Vector2 = mapmake1.Vector2

def nameIncremental(i):
	alphabet = 'abcdefghijklmnopqrstuvwxyz'
	output = ''
	while i >= 26:
		i, p = divmod(i, 26)
		output += alphabet[p]
	#loop
	output += alphabet[i]
	return output
#end def nameIncremental(i)

def createMap3(nodes=8, branches=7, seed_id=3243):
	"""
	Create a map with <nodes> backbone nodes and <branches> branches.
	"""
	mapmake1.seed(seed_id)
	a = mapmake1.Map()
	b = []
	for i in range(nodes):
		b.append(mapmake1.Point(nameIncremental(i)))
	#next i
	# Connect the backbone end to end a--b--...--n
	for i in range(nodes-1):
		b[i].addConn(b[i+1])
	#next i
	for i in range(branches):
		addBranch1 = mapmake1.randint(0,len(b)-1)
		b.append(mapmake1.Point(nameIncremental(len(b))))
		b[addBranch1].addConn(b[-1])
	#next i
	a.addPoints(b)
	# This is the magic function I just wrote.
	a.sortBranch()
	a.centerBound()
	return a
#end def createMap3([nodes], [conns], [seed_id])

def main():
	import sys
	import os
	
	nodes = 20
	conns = 4
	if len(sys.argv) > 1:
		nodes = int(sys.argv[1])
		if len(sys.argv) > 2:
			branches = int(sys.argv[2])
		#end if
	#end if
	
	map1 = createMap3(nodes, branches, 9999)
	map1.desc = "SWW Optimize map with {} nodes and {} branches".format(nodes, branches)
	#dri = map1.points[0] - map1.points[1]

	# FIXME: Race condition is very likely due to multiple programs running at once.
	file_num = 0
	while os.path.exists('sww_brute3_%i.svg' % file_num):
		file_num += 1
	#loop
	print('sww_brute3_%i.svg' % file_num)
	open('sww_brute3_%i.svg' % file_num, 'w').write(str(map1))
	open('sww_brute3_%i.json' % file_num, 'w').write(map1.saveJson())
#end def main()

if __name__ == '__main__':
    main()
#end if
